import java.util.ArrayList;
import java.util.Random;

import weka.core.Instance;
import weka.core.Instances;
import weka.core.converters.ConverterUtils.DataSource;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.Remove;

public class KMeans {
	// Here the number of clusters K
	private static final int NUM_CLUSTERS = 2; // This is K
	// Here the original data
	private static Instances data;
	// Here the data with only two dimensions
	private static Instances dataCluster;

	private static ArrayList<Data> dataSet = new ArrayList<Data>();
	private static ArrayList<Centroid> centroids = new ArrayList<Centroid>();

	private static void initialize() {
		Random r = new Random(213214123);
		for (int i = 0; i < NUM_CLUSTERS; ++i) {
			int rand = r.nextInt(dataCluster.numInstances());
			Instance randomPoint = dataCluster.instance(rand);
			centroids.add(new Centroid(randomPoint.value(0), randomPoint
					.value(1)));
		}
		System.out.println("Centroids initialized at:");
		for (Centroid c : centroids) {
			System.out.println("     (" + c.X() + ", " + c.Y() + ")");
		}
		System.out.print("\n");
		return;
	}

	private static void kMeanCluster() {
		// some big number that's sure to be larger than our data range.
		final double bigNumber = Float.MAX_VALUE;
		double minimum = bigNumber; // The minimum value to beat.
		double distance = 0.0; // The current minimum value.
		int sampleNumber = 0;
		int cluster = 0;
		boolean isStillMoving = true;
		Data newData = null;

		// Add in new data, one at a time, recalculating centroids with each new
		// one.
		while (dataSet.size() < dataCluster.numInstances()) {
			newData = new Data(dataCluster.instance(sampleNumber).value(0),
					dataCluster.instance(sampleNumber).value(1),
					dataCluster.instance(sampleNumber), dataCluster.instance(
							sampleNumber).stringValue(2));
			dataSet.add(newData);
			minimum = bigNumber;
			for (int i = 0; i < NUM_CLUSTERS; i++) {
				distance = dist(newData, centroids.get(i));
				if (distance < minimum) {
					minimum = distance;
					cluster = i;
				}
			}
			newData.cluster(cluster);
			sampleNumber++;
		}

		// Now, keep shifting centroids until equilibrium occurs.
		while (isStillMoving) {
			// calculate new centroids.
			for (int i = 0; i < NUM_CLUSTERS; i++) {
				int totalX = 0;
				int totalY = 0;
				int totalInCluster = 0;
				for (int j = 0; j < dataSet.size(); j++) {
					if (dataSet.get(j).cluster() == i) {
						totalX += dataSet.get(j).X();
						totalY += dataSet.get(j).Y();
						totalInCluster++;
					}
				}
				if (totalInCluster > 0) {
					centroids.get(i).X(
							((double) totalX / (double) totalInCluster));
					centroids.get(i).Y(
							((double) totalY / (double) totalInCluster));
				}
			}

			// Assign all data to the new centroids
			isStillMoving = false;
			for (int i = 0; i < dataSet.size(); i++) {
				Data tempData = dataSet.get(i);
				minimum = bigNumber;
				for (int j = 0; j < NUM_CLUSTERS; j++) {
					distance = dist(tempData, centroids.get(j));
					if (distance < minimum) {
						minimum = distance;
						cluster = j;
					}
				}
				tempData.cluster(cluster);
				if (tempData.cluster() != cluster) {
					tempData.cluster(cluster);
					isStillMoving = true;
				}
			}
		}
		return;
	}

	/**
	 * Calculate Euclidean distance.
	 * 
	 * @param d
	 *            - Data object.
	 * @param c
	 *            - Centroid object.
	 * @return - double value.
	 */
	private static double dist(Data d, Centroid c) {
		return Math.sqrt(Math.pow((c.Y() - d.Y()), 2)
				+ Math.pow((c.X() - d.X()), 2));
	}

	private static class Data {
		private double mX = 0;
		private double mY = 0;
		private Instance refersTo;
		private String label = "";
		private int mCluster = 0;

		public Data(double x, double y, Instance ref, String label) {
			this.X(x);
			this.Y(y);
			this.refersTo = ref;
			this.label = label;
		}

		public void X(double x) {
			this.mX = x;
		}

		public double X() {
			return this.mX;
		}

		public void Y(double y) {
			this.mY = y;
		}

		public double Y() {
			return this.mY;
		}

		public void setLabel(String label) {
			this.label = label;
		}

		public String getLabel() {
			return label;
		}

		public void setRefersTo(Instance to) {
			this.refersTo = to;
		}

		public Instance getRefersTo() {
			return this.refersTo;
		}

		public void cluster(int clusterNumber) {
			this.mCluster = clusterNumber;
		}

		public int cluster() {
			return this.mCluster;
		}
	}

	private static class Centroid {
		private double mX = 0.0;
		private double mY = 0.0;

		public Centroid(double newX, double newY) {
			this.mX = newX;
			this.mY = newY;
		}

		public void X(double newX) {
			this.mX = newX;
		}

		public double X() {
			return this.mX;
		}

		public void Y(double newY) {
			this.mY = newY;
		}

		public double Y() {
			return this.mY;
		}
	}

	public static void main(String[] args) {
		loadDataset("iris.arff");
		initialize();
		kMeanCluster();

		// Print out clustering results.
		for (int i = 0; i < NUM_CLUSTERS; i++) {
			System.out.println("Cluster " + i + " includes:");
			for (int j = 0; j < dataCluster.numInstances(); j++) {
				if (dataSet.get(j).cluster() == i) {
					System.out.println("     (" + dataSet.get(j).X() + ", "
							+ dataSet.get(j).Y() + ")");
				}
			}
			System.out.println();
		}

		// Print out centroid results.
		System.out.println("Centroids finalized at:");
		for (int i = 0; i < NUM_CLUSTERS; i++) {
			System.out.println("     (" + centroids.get(i).X() + ", "
					+ centroids.get(i).Y() + ")");
		}
		System.out.print("\n");
		return;
	}

	/**
	 * Load a dataset in ARFF format
	 * 
	 * @param arffPath
	 */
	private static void loadDataset(String arffPath) {
		try {
			// ORIGINAL DATA
			DataSource source = new DataSource(arffPath);
			data = source.getDataSet();
			// Set the class Attribute index
			if (data.classIndex() == -1)
				data.setClassIndex(data.numAttributes() - 1);

			DataSource source1 = new DataSource(arffPath);
			dataCluster = source1.getDataSet();
			// Set the class Attribute index
			if (dataCluster.classIndex() == -1)
				dataCluster.setClassIndex(dataCluster.numAttributes() - 1);

			while (dataCluster.numAttributes() > 3) {
				Remove filter = new Remove();
				filter.setAttributeIndices("" + (dataCluster.classIndex()));
				filter.setInputFormat(dataCluster);
				dataCluster = Filter.useFilter(dataCluster, filter);
				dataCluster.setClassIndex(dataCluster.numAttributes() - 1);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
